# Modules used to import data into the knowledge base.

# Copyright (C) 2018, 2019 Nathan Matthew Miller

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from os import listdir
from os import stat
from datetime import datetime
from os.path import isfile, join, basename
import re

import KB.Core as core


# Return creation, last modification times, and possibly a title for
# a file.
# On Unix, since creation time is when the last metadata change occurred,
# check and see if the file's name encodes the creation time:
# An example of such a name is "2014-11-21T09-13-58.org".
#
# Some of org-mode files use the filename to hold the title of the entry, if
# that's the case then return the file's name as the title.
#
def get_file_info( file ):

    date_for_filename = False
    
    st = stat( file )
    mtime = datetime.fromtimestamp( st.st_mtime ).isoformat()

    creation_match = re.search( '(\d{4}-\d\d-\d\dT\d\d-\d\d-\d\d)\.org$',
                                file )
    if creation_match == None:

        ctime = datetime.fromtimestamp( st.st_ctime ).isoformat()
    else:
        date_for_filename = True
        time = creation_match[1]

        # Fix up the string to replace '-' with ':' in the time portion.
        i = time.rfind( 'T' )
        date = time[:i]
        fixed_time = time[i:].replace( '-', ':' )
        ctime = date + fixed_time

    title = None
    if not date_for_filename and \
       re.search( 'deft-\d+\.org$', file ) == None:
       m = re.search( '(.+?)\.org$', basename(file) )
       if m != None:
           title = m[1].capitalize()
        
    return (ctime, mtime, title)


# Import all files in a directory and assume the files are org-mode
# formatted files.
#
# Arguments: kb - a KB_Core instance; dir - directory to import from.
# Return: A list of files which have issues being imported.
#
# TODO: make the import more sophisticated, parse out links, etc.
#
def org_import_dir( kb, dir ):

    files  = [f for f in listdir( dir ) if isfile( join( dir, f ))]
    print( f"Found {len(files)} files to import." )
    
    errors = []
    count = 0
    
    # Used to remove "* " org-mode formatting from the title.
    strip_title = re.compile( "^[\*\s]*(.+)$" )
    
    for file in files:

        full_path = join( dir, file )
        (ctime, mtime, title) = get_file_info( full_path )
        
        with open( full_path, 'r', encoding='utf-8' ) as f:
            
            frag = None

            try:
                if title == None:
                    title = f.readline().rstrip()
                    m = strip_title.match( title )
                    if  m != None:
                        title = m[1]
                
                frag = f.read()

                kb.kf_create( name=title, content=frag,
                              created =ctime,
                              modified=mtime )
                count += 1
                    
            except UnicodeDecodeError as e:

                errors.append( (file, str(e)) )

    print( f"Imported {count} files." )

    return errors
