# Copyright (C) 2018, 2019 Nathan Matthew Miller

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import tkinter as tk
from tkinter import messagebox

from TkGUI.Editor import *

class Controller:

    def __init__( self, kb, top, link_id ):

        self.kb      = kb
        self.top     = top
        self.link_id = link_id
        
        self.entity      = None
        self.entity_id   = None
        self.entity_type = None
        return


    def set_model( self, model ):

        self.model = model
        return

    
    def load_entity( self, entity_id ):

        self.entity_id = entity_id
        
        entity_details = self.kb.retrieve_entity( entity_id )

        self.entity_type = entity_details[0]
        e_name = entity_details[1]

        relations = self.kb.retrieve_relations( entity_id )
        self.model.load_entity( e_name, entity_details, relations )
        return


    # Save the loaded entity to the KB.
    #
    def save_entity( self ):

        content = self.model.get_content()
        title   = self.model.get_title()
        
        try:
            if self.entity_type == 'K':

                self.kb.kf_update( self.entity_id, content=content,
                                   name=title )
                print( "Saved KF " + self.entity_id )
            else:
                self.kb.category_update( self.entity_id, title )

                print( "Saved CAT " + self.entity_id )
        except Exception as error:

            messagebox.showerror( 'Save Error', str(error), parent=self.top,
                                    default=messagebox.OK )
        return
    

    # Event handler for the Text widget.
    #
    def pre_handler( self, event ):

        rtn = None
        
        # Has a range been selected?
        ranges = event.widget.tag_ranges( "sel" )
        if ranges:
            self.model.set_range( *ranges )
        else:
            self.model.clear_range()
        
        if event.type == '2': # and /
           # not ((0x0004 | 0x0008 | 0x0010) & event.state):
            # a KeyPress without a Ctrl or Alt modifier pressed as well.
            index = event.widget.index( tk.INSERT )
            rtn = self.model.key_press( index, event )
        
        return rtn


    # When a <<Paste>> event occurs the model must be updated correctly to
    # match the Text widget contents.
    #
    def paste_handler( self, event ):

        self.model.delete_selection( event.widget )
        self.model.insert_paste( event.widget )
        return


    # When a <<Cut>> event occurs, remove the selected text from the model.
    #
    def cut_handler( self, event ):

        self.model.delete_selection( event.widget )
        return

    

    # Store the ID so that it can later be linked to another entity.
    #
    def store_entity_id( self, event ):

        self.link_id.set( self.entity_id )
        print( f"Stored link: {self.entity_id}" )
        return


    # An entity ID has previously been stored, link that ID to this entity.
    #
    def link_to_entity( self, event ):
        
        try:
            link = self.link_id.get()
            assert len(link) > 0, "No entity link present"

            print( f"{self.entity_id} <-- {link}")
            
            entity = self.kb.retrieve_entity( link )
            self.kb.relation_create( parent=self.entity_id, child=link,
                                     directed=True )
            self.model.add_relationship_link( link, entity )
        
        except Exception as error:
            messagebox.showerror( "Error!", str(error), parent=self.top )
            
        return


    # Add a stored entity ID as an entry to a category.
    #
    def add_to_category( self, event ):

        try:
            link = self.link_id.get()
            assert len(link) > 0, "No entity link present"
            
            entity = self.kb.retrieve_entity( link )
            self.kb.category_add( category_id=self.entity_id,
                                  entity_id=link )
            self.model.category_add( link, entity )
        
        except Exception as error:
            messagebox.showerror( "Error!", str(error), parent=self.top )
        
        return
    

    def remove_relation( self, relation_id ):

        try:
            self.kb.relation_remove( parent=self.entity_id, child=relation_id )
        except Exception as error:
            messagebox.showerror( "Error!", str(error), parent=self.top )

        return


    def remove_from_category( self, remove_id ):

        self.kb.category_remove( self.entity_id, remove_id )
        return


    # Load an existing entity.
    #
    def show_entity( self, id ):

        import TkGUI

        assert id != None, "Undefined entity ID"
        TkGUI.Editor.Editor( self.kb, self.link_id, entity_id = id )
        return
